def create(keyword, page_no, page_cnt):
    operation_name = "getRestaurants"
    variables = _define_variables(keyword, page_no, page_cnt)
    query = {
        "operationName": operation_name,
        "variables": variables,
        "query": QUERY_STATEMENT
    }
    return query


def _define_variables(keyword: str, page_no: int, page_cnt: int):
    return {
        "input": {"query": keyword, "display": page_cnt, "start": page_no, "isNmap": False, "deviceType": "pcmap"},
        "isNmap": False,
        "isBounds": True
    }


QUERY_STATEMENT = """query getRestaurants(
                        $input: RestaurantsInput
                        $isNmap: Boolean!
                        $isBounds: Boolean!
                        ) {
                        restaurants(input: $input) {
                            total
                            items {
                            ...RestaurantItemFields
                            easyOrder {
                                easyOrderId
                                easyOrderCid
                                businessHours {
                                weekday {
                                    start
                                    end
                                    __typename
                                }
                                weekend {
                                    start
                                    end
                                    __typename
                                }
                                __typename
                                }
                                __typename
                            }
                            baemin {
                                businessHours {
                                deliveryTime {
                                    start
                                    end
                                    __typename
                                }
                                closeDate {
                                    start
                                    end
                                    __typename
                                }
                                temporaryCloseDate {
                                    start
                                    end
                                    __typename
                                }
                                __typename
                                }
                                __typename
                            }
                            yogiyo {
                                businessHours {
                                actualDeliveryTime {
                                    start
                                    end
                                    __typename
                                }
                                bizHours {
                                    start
                                    end
                                    __typename
                                }
                                __typename
                                }
                                __typename
                            }
                            __typename
                            }
                            nlu {
                            ...NluFields
                            __typename
                            }
                            brand {
                            name
                            isBrand
                            type
                            menus {
                                order
                                id
                                images {
                                url
                                desc
                                __typename
                                }
                                name
                                desc
                                price
                                isRepresentative
                                detailUrl
                                orderType
                                catalogId
                                source
                                menuId
                                nutrients
                                allergies
                                __typename
                            }
                            __typename
                            }
                            optionsForMap @include(if: $isBounds) {
                            maxZoom
                            minZoom
                            includeMyLocation
                            maxIncludePoiCount
                            center
                            spotId
                            __typename
                            }
                            __typename
                        }
                        }
                        fragment RestaurantItemFields on RestaurantSummary {
                        id
                        dbType
                        name
                        businessCategory
                        category
                        description
                        hasBooking
                        hasNPay
                        x
                        y
                        distance
                        imageUrl
                        imageUrls
                        imageCount
                        phone
                        virtualPhone
                        routeUrl
                        streetPanorama {
                            id
                            pan
                            tilt
                            lat
                            lon
                            __typename
                        }
                        roadAddress
                        address
                        commonAddress
                        blogCafeReviewCount
                        bookingReviewCount
                        totalReviewCount
                        bookingReviewScore
                        bookingUrl
                        bookingHubUrl
                        bookingHubButtonName
                        bookingBusinessId
                        talktalkUrl
                        options
                        promotionTitle
                        agencyId
                        businessHours
                        microReview
                        tags
                        priceCategory
                        broadcastInfo {
                            program
                            date
                            menu
                            __typename
                        }
                        michelinGuide {
                            year
                            star
                            comment
                            url
                            hasGrade
                            isBib
                            alternateText
                            __typename
                        }
                        broadcasts {
                            program
                            menu
                            episode
                            broadcast_date
                            __typename
                        }
                        tvcastId
                        naverBookingCategory
                        saveCount
                        uniqueBroadcasts
                        isDelivery
                        isCvsDelivery
                        markerLabel @include(if: $isNmap) {
                            text
                            style
                            __typename
                        }
                        imageMarker @include(if: $isNmap) {
                            marker
                            markerSelected
                            __typename
                        }
                        isTableOrder
                        isPreOrder
                        isTakeOut
                        bookingDisplayName
                        bookingVisitId
                        bookingPickupId
                        popularMenuImages {
                            name
                            price
                            bookingCount
                            menuUrl
                            menuListUrl
                            imageUrl
                            isPopular
                            usePanoramaImage
                            __typename
                        }
                        visitorReviewCount
                        visitorReviewScore
                        detailCid {
                            c0
                            c1
                            c2
                            c3
                            __typename
                        }
                        streetPanorama {
                            id
                            pan
                            tilt
                            lat
                            lon
                            __typename
                        }
                        __typename
                        }
                        fragment NluFields on Nlu {
                        queryType
                        user {
                            gender
                            __typename
                        }
                        queryResult {
                            ptn0
                            ptn1
                            region
                            spot
                            tradeName
                            service
                            selectedRegion {
                            name
                            index
                            x
                            y
                            __typename
                            }
                            selectedRegionIndex
                            otherRegions {
                            name
                            index
                            __typename
                            }
                            property
                            keyword
                            queryType
                            nluQuery
                            businessType
                            cid
                            branch
                            franchise
                            titleKeyword
                            location {
                            x
                            y
                            default
                            longitude
                            latitude
                            dong
                            si
                            __typename
                            }
                            noRegionQuery
                            priority
                            showLocationBarFlag
                            themeId
                            filterBooking
                            repRegion
                            repSpot
                            dbQuery {
                            isDefault
                            name
                            type
                            getType
                            useFilter
                            hasComponents
                            __typename
                            }
                            type
                            category
                            menu
                            __typename
                        }
                        __typename
                        }
                        """



import json
import re
from urllib.parse import urlencode

import pandas as pd
import time
from selenium.common import NoSuchElementException
from seleniumwire import webdriver as wired_webdriver
from selenium import webdriver
from seleniumwire import webdriver, utils
import requests
from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.service import Service
from webdriver_manager.chrome import ChromeDriverManager

from rotate_ip_aws_gw.ip_rotator import ApiGateway
from . import query
from .utils import HEADERS, parse_naver_var_in_script_texts


def wait_for_graphql_response(driver,
                              max_attempts=10,
                              waiting_time=2
                              ):
    current_attempt = 0

    while current_attempt < max_attempts:
        need_to_waiting = False
        for request in driver.requests:
            request_url = request.url
            if "graphql" not in request_url:
                continue
            if request.response is None:
                need_to_waiting = True
                break
        if need_to_waiting == False:
            break
        time.sleep(waiting_time)


def get_value_by_path(data, path):
    keys = path.split('.')  # 경로를 점을 기준으로 분할하여 리스트 생성
    result = data

    for key in keys:
        if isinstance(result, dict) and key in result:
            result = result[key]
        else:
            return None  # 경로를 따라가는 동안 오류 발생 시 None 반환

    return result


def extract_naver_var(html_text):
    soup = BeautifulSoup(html_text, "html.parser", from_encoding="utf-8")
    scripts = soup.find_all("script")

    naver_var = parse_naver_var_in_script_texts(scripts)
    variable_name = 'window.__APOLLO_STATE__'
    pattern = re.compile(rf'\b{re.escape(variable_name)}\s*=\s*(.*?)(\n)')
    match = pattern.search(naver_var)

    if match is None:
        return None
    # 첫페이지
    data = match.group(1)
    json_data = json.loads(data[:-1])
    return json_data


def collect_graphql_responses(driver):
    responses = []
    for request in driver.requests:
        request_url = request.url
        if "graphql" not in request_url:
            continue
        response = utils.decode(
            request.response.body,
            request.response.headers.get("Content-Encoding", "identity")
        )
        response = response.decode("utf8")
        json_data = json.loads(response)
        responses.append(json_data)
    return responses


class PlaceMixin:
    def search_places(self, keyword: str, page_no: int, page_size: int, proxies=None):
        data = query.get_restaurants.create(keyword, page_no, page_size)
        response = requests.post("https://pcmap-api.place.naver.com/graphql", headers=HEADERS, data=json.dumps(data),
                                 proxies=proxies)
        response.raise_for_status()
        response_data = response.json()
        graphql_data = response_data['data']['restaurants']
        return graphql_data

    def _get_tor_session(self):
        # initialize a requests Session
        session = requests.Session()
        # this requires a running Tor service in your machine and listening on port 9050 (by default)
        session.proxies = {"http": "socks5://localhost:9150", "https": "socks5://localhost:9150"}
        # session.proxies = {'http': 'http://67.43.227.229:29003', 'https':'https://67.43.227.229:29003'}
        return session

    def get_place(self,
                  business_id,
                  use_tor:bool=True,
                  tor_port:int=9150,
                  proxies=None):
        # url = f"https://pcmap.place.naver.com/restaurant/{business_id}/home"
        url = "https://pcmap.place.naver.com/restaurant/{}".format(business_id)

        if use_tor:
            session = self._get_tor_session()
            ip = session.get("http://icanhazip.com").text
            response = session.get(url)
            # proxies = {"http": f"socks5://localhost:{tor_port}", "https": f"socks5://localhost:{tor_port}"}
        else:
            response = requests.get(url, proxies=proxies)
        response.raise_for_status()
        html_text = response.content

        broadcasts = self.__parse_broadcast_in_html(html_text)
        return broadcasts

    def get_restaurant(self, business_id, proxies):
        data = query.get_restaurant.create(business_id)
        response = requests.post("https://pcmap-api.place.naver.com/graphql", headers=HEADERS, data=json.dumps(data),
                                 proxies=proxies)
        response.raise_for_status()
        response_data = response.json()
        graphql_data = response_data['data']
        return graphql_data

    def __parse_broadcast_in_html(self, html_text: str):
        soup = BeautifulSoup(html_text, "html.parser", from_encoding="utf-8")
        scripts = soup.find_all("script")

        naver_var = parse_naver_var_in_script_texts(scripts)
        variable_name = 'window.__APOLLO_STATE__'
        pattern = re.compile(rf'\b{re.escape(variable_name)}\s*=\s*(.*?)(\n)')
        match = pattern.search(naver_var)

        if match:
            data = match.group(1)
            return json.loads(data[:-1])
        else:
            return None

    def get_place_summary(self, id: str, proxies=None):
        url = f"https://map.naver.com/v5/api/sites/summary/" + id + "?lang=ko"
        data = {
            "language": 'kor',
            "order_by": "2"
        }
        response = requests.get(url, data=data, proxies=proxies)
        response.raise_for_status()
        resposne_data = response.json()
        resposne_data['_id'] = id
        return resposne_data

    def get_menus(self, business_id, proxies=None):
        url = f"https://pcmap.place.naver.com/restaurant/{business_id}/menu/list"
        response = requests.get(url, proxies=proxies)
        response.raise_for_status()
        html_text = response.content
        menus = self.__parse_menus_internal(html_text)
        result = {
            "business_id": business_id,
            "menus": menus
        }
        return result

    def __parse_menus_internal(self, html_text: str):
        soup = BeautifulSoup(html_text, "html.parser", from_encoding="utf-8")
        scripts = soup.find_all("script")
        naver_string = None

        for s in scripts:
            if s.string is None:
                continue
            if "var naver=" in s.string:
                naver_string = s.string

        menu_regex = re.compile(r'\{"__typename":"Menu".*?\}')
        menu_string_list = re.findall(menu_regex, naver_string)

        # 일반
        menus = []
        for menu_string in menu_string_list:
            menu_json = json.loads(menu_string)
            menu = {
                "name": menu_json["name"],
                "price": menu_json["price"],
                "description": menu_json["description"],
                "imageUrl": menu_json["images"][0] if menu_json["images"] else "",
                "isRecommended": menu_json["recommend"],
            }
            menus.append(menu)
        # baemin
        baemin_menu_regex = re.compile(r'\{"__typename":"BaeminMenu".*?\}')
        baemin_menu_string_list = re.findall(baemin_menu_regex, naver_string)

        for baemin_menu_string in baemin_menu_string_list:
            baemin_menu_json = json.loads(baemin_menu_string)
            menu = {
                "name": baemin_menu_json["name"],
                "price": baemin_menu_json["price"],
                "imageUrl": menu_json["images"][0] if menu_json["images"] else "",

            }
            menus.append(menu)
        return menus

    def search_keyword_in_html(self,
                               keyword, # 관심지역내 검색 키워드
                               x=None,  # 관심지역내 검색 중심 위치 좌표
                               y=None,  # 관심지역내 현재 중심 위치 좌표
                               bounds=None,   # 관심지역 좌표영역(좌하단 좌표, 우상단 좌표)
                               is_current_location_search=True, # 주변 지역 검색 포함 여부
                               is_headless=True,  # Selenium, 브라우저 출력 여부
                               interval_page=3,
                               use_tor= False,
                               tor_port= 9150,
                               aws_api_gw_endpoint=None,
                               proxies=None): # 프록시 서버 사용 여부
        params = {
            'query': keyword,
            "x": x,
            'y': y,
            'clientX': x,
            'clientY': y,
            'bounds': bounds,
            'isCurrentLocationSearch': is_current_location_search
        }
        encoded_params = urlencode(params)
        base_url = "https://pcmap.place.naver.com/restaurant/list"
        url = base_url + '?' + encoded_params

        options = webdriver.ChromeOptions()
        options.add_argument('--disable-dev-shm-usage')
        if is_headless is True:
            options.add_argument('headless')
            options.add_argument('--no-sandbox')
        seleniumwire_options = {}
        if use_tor is True:
            seleniumwire_options['proxy'] = {
                'http':f'socks5://127.0.0.1:{tor_port}',
                'https':f'socks5://127.0.0.1:{tor_port}',
                'no_proxy': 'localhost,127.0.0.1'
            }
        driver = wired_webdriver.Chrome(service=Service(ChromeDriverManager().install()),
                                        options=options,
                                        seleniumwire_options=seleniumwire_options)
        driver.get("http://icanhazip.com")
        html_text = driver.page_source
        try:
            if aws_api_gw_endpoint is None:

                def request_interceptor(request):
                    if "cc.naver.com" in request.host:
                        request.abort()
                    if "sentry" in request.host:
                        request.abort()
                    if "search.pstatic.net" in request.host:
                        request.abort()
                    if request.path.endswith(('.png', '.jpg', '.gif', 'jpeg', "css")):
                        request.abort()

                driver.request_interceptor = request_interceptor
                driver.get(url)
            else:
                gw = ApiGateway(site='')
                gw.getAsSelenium(driver, url, aws_api_gw_endpoint)
        except Exception as e:
            driver.quit()
            return {
                'html_text': None,
                'restaurants': [],
                'ad_business': [],
                'naver_var': None
            }
        html_text = driver.page_source
        try:
            pagination_element = driver.find_element(By.CLASS_NAME, "zRM9F")
        except NoSuchElementException as e:
            driver.quit()
            return {
                'html_text': html_text,
                'restaurants': [],
                'ad_business': [],
                'naver_var': None
            }
        child_elements = pagination_element.find_elements(By.XPATH, "./*")

        def move_second_to_last(arr):
            if len(arr) >= 2:
                second_element = arr.pop(1)
                arr.append(second_element)
            return arr

        # 두 번째 요소를 가져옵니다.
        child_elements = move_second_to_last(child_elements)

        for child_element in child_elements[1:]:
            try:
                if child_element.aria_role != 'button':
                    continue
                child_element.click()
                time.sleep(interval_page)
            except NoSuchElementException as e:
                pass

        wait_for_graphql_response(driver)
        graphql_responses = collect_graphql_responses(driver)
        naver_var = extract_naver_var(html_text)

        restaurants = []
        ad_business = []
        for graphql_response in graphql_responses:
            for response in graphql_response:
                restaurants_response = get_value_by_path(response, "data.restaurants")
                if restaurants_response is not None:
                    restaurants.append(restaurants_response)

                ad_business_response = get_value_by_path(response, "data.adBusinesses")
                if ad_business_response is not None:
                    ad_business.append(ad_business_response)
        driver.close()
        return {
            'html_text': html_text,
            'restaurants': restaurants,
            'ad_business': ad_business,
            'naver_var': naver_var
        }

    def search_keyword_in_html_in_first_page(self, keyword, x=None, y=None, bounds=None, proxies=None):
        url = "https://pcmap.place.naver.com/restaurant/list?query={}".format(keyword)
        # url = 'https://map.naver.com/v5/search/'
        response = requests.get(url,
                                params={
                                    "x": x,
                                    'y': y,
                                    'clientX': x,
                                    'clientY': y,
                                    'bounds': bounds
                                },
                                proxies=proxies)
        response.raise_for_status()
        html_text = response.content
        soup = BeautifulSoup(html_text, "html.parser", from_encoding="utf-8")
        scripts = soup.find_all("script")

        naver_var = parse_naver_var_in_script_texts(scripts)
        variable_name = 'window.__APOLLO_STATE__'
        pattern = re.compile(rf'\b{re.escape(variable_name)}\s*=\s*(.*?)(\n)')
        match = pattern.search(naver_var)

        if match is None:
            return None

        data = match.group(1)
        json_data = json.loads(data[:-1])

        return json_data
    #
    #
    # def search_keyword_in_bound(self, keyword, x, y, bound):



import json
import urllib
import urllib.parse
import urllib.request


class SearchMixin:
    CLIENT_ID = "ak4jckPbdjMcrMmFDSgv"
    CLIENT_SECRET = "Ye3VCcl2in"
    def all_search(self,
                  query,
                  type,  # ex) all/place
                  searchCoord, #    ex. 126.942428;37.485309
                  boundary = None
                  ):
        encoded_keyword = urllib.parse.quote(query)
        # url = f"https://openapi.naver.com/v1/search/local.json?query={encoded_keyword}&display={page_size}"
        url = f"https://map.naver.com/p/api/search/allSearch?query={encoded_keyword}&type={type}&searchCoord={searchCoord}&boundary={boundary}"
        request = urllib.request.Request(url)
        response = urllib.request.urlopen(request)
        rescode = response.getcode()

        if rescode == 200:
            response_body = response.read()
            results = json.loads(response_body)
            return results['result']
        else:
            return None
    def search(self,
               keyword: str, page_size: int = 5,
               start:int = 10,
               client_id: str = CLIENT_ID, client_secret: str = CLIENT_SECRET,
               proxies=None):
        encoded_keyword = urllib.parse.quote(keyword)
        url = f"https://openapi.naver.com/v1/search/local.json?query={encoded_keyword}&display={page_size}&start=5"

        request = urllib.request.Request(url)
        request.add_header("X-Naver-Client-Id", client_id)
        request.add_header("X-Naver-Client-Secret", client_secret)
        response = urllib.request.urlopen(request)
        rescode = response.getcode()

        result_list = []
        if rescode == 200:
            response_body = response.read()
            results = json.loads(response_body)['items']

            for result in results:
                result['title'] = result['title'].replace('<b>', '').replace('</b>', '')
                result_list.append(result)
        return result_list
    def search_blog(self,
               keyword: str, page_size: int = 5,
               client_id: str = CLIENT_ID, client_secret: str = CLIENT_SECRET,
               proxies=None):
        encoded_keyword = urllib.parse.quote(keyword)
        url = f"https://openapi.naver.com/v1/search/blog.json?query={encoded_keyword}&display={page_size}"

        request = urllib.request.Request(url)
        request.add_header("X-Naver-Client-Id", client_id)
        request.add_header("X-Naver-Client-Secret", client_secret)
        response = urllib.request.urlopen(request)
        rescode = response.getcode()

        result_list = []
        if rescode == 200:
            response_body = response.read()
            results = json.loads(response_body)['items']

            for result in results:
                result['title'] = result['title'].replace('<b>', '').replace('</b>', '')
                result_list.append(result)
        return result_list



def create(business_id: str, page_no: int, page_size: int):
    operation_name = 'getFsasReviews'
    variables = _define_variables(business_id, page_no, page_size)
    query = {
        "operationName": operation_name,
        "variables": variables,
        "query": QUERY_STATEMENT
    }
    return query


def _define_variables(business_id: str, page_no: int, page_size: int):
    return {
            "input":
                {
            "businessId": business_id,
            'businessType': 'restaurant',
            'deviceType': 'mobile',
            "display": page_size,
            "page": page_no
        }
    }


QUERY_STATEMENT = 'query getFsasReviews($input: FsasReviewsInput) {\n  fsasReviews(input: $input) {\n    ...FsasReviews\n    __typename\n  }\n}\n\nfragment FsasReviews on FsasReviewsResult {\n  total\n  maxItemCount\n  items {\n    name\n    type\n    typeName\n    url\n    home\n    id\n    title\n    rank\n    contents\n    bySmartEditor3\n    hasNaverReservation\n    thumbnailUrl\n    thumbnailUrlList\n    thumbnailCount\n    date\n    isOfficial\n    isRepresentative\n    profileImageUrl\n    isVideoThumbnail\n    reviewId\n    authorName\n    createdString\n    bypassToken\n    __typename\n  }\n  __typename\n}'



def create(id: str):
    query = {
        "operationName": "getVisitorReviewStats",
        "variables": {
            "businessType": "restaurant",
            "id": id
        },
        "query": QUERY_STATEMENT
    }
    return query


def _define_variables(id: str):
    return {
        "id": id,
        "businessType": "restaurant",
    }


QUERY_STATEMENT = """query getVisitorReviewStats($id: String, $itemId: String, $businessType: String = "place") {
                                visitorReviewStats(input: {businessId: $id, itemId: $itemId, businessType: $businessType}) {
                                    id
                                    name
                                    review {
                                        avgRating
                                        totalCount
                                        starDistribution {
                                            count
                                            score
                                        }
                                        imageReviewCount
                                        authorCount
                                    }
                                    analysis {
                                        themes {
                                            code
                                            label
                                            count
                                        }
                                        menus {
                                            label
                                            count
                                        }
                                        votedKeyword {
                                            totalCount
                                            reviewCount
                                            userCount
                                            details {
                                                category
                                                code
                                                iconUrl
                                                iconCode
                                                displayName
                                                count
                                                previousRank
                                            }
                                        }
                                    }
                                    visitorReviewsTotal
                                    ratingReviewsTotal
                                }
                            }"""



import requests

from .utils import HEADERS


class CategoryMixin:
    def get_categories(self,
                       keyword: str,
                       search_use: bool = True,
                       nmb_use: bool = True,
                       only_leaf: bool = True,
                       project_place_brand: bool = True,
                       is_use_keyword_like_match: bool = False,
                       proxies=None):
        parameters = {
            'search': keyword,
            'searchUse': search_use,
            'nmbUse': nmb_use,
            'onlyLeaf': only_leaf,
            'projectPlaceBrand': project_place_brand,
            'isUseKeywordLikeMatch': is_use_keyword_like_match
        }
        response = requests.get("https://new.smartplace.naver.com/api/pbp/public/categories",
                                headers=HEADERS,
                                params=parameters,
                                proxies=proxies)
        response.raise_for_status()
        response_data = response.json()
        return response_data



import re
import json
import requests
from bs4 import BeautifulSoup

from .utils import HEADERS, parse_naver_var_in_script_texts


class BroadcastMixin:
    def get_broadcasts_of_store(self, business_id, proxies=None):
        # url = f"https://pcmap.place.naver.com/restaurant/{business_id}/home"
        url = "https://pcmap.place.naver.com/restaurant/{}".format(business_id)
        response = requests.get(url, proxies=proxies)
        response.raise_for_status()
        html_text = response.content

        broadcasts = self.__parse_broadcast_in_html(html_text)
        result = {
            "business_id": business_id,
            "broadcasts": broadcasts
        }
        return result

    def __parse_broadcast_in_html(self, html_text: str):
        soup = BeautifulSoup(html_text, "html.parser", from_encoding="utf-8")
        scripts = soup.find_all("script")

        naver_var = parse_naver_var_in_script_texts(scripts)

        # broadcast
        media_regex = re.compile(r'\{"__typename":"BroadcastInfo".*?\}')
        media_string_list = re.findall(media_regex, naver_var)

        broadcasts = []
        for media_string in media_string_list:
            media_json = json.loads(media_string)
            broadcast = {
                "channel": media_json["channel"],
                "program": media_json["program"],
                "episode": media_json["episode"],
                "date": media_json["date"],
                "menu": media_json["menu"],
                "tvcast": media_json["tvcast"]
            }
            broadcasts.append(broadcast)

        return broadcasts



import json
import math
import re

from selenium.common import NoSuchElementException, StaleElementReferenceException
from seleniumwire import webdriver, utils
from selenium import webdriver
import time
import requests
from bs4 import BeautifulSoup
from seleniumwire import webdriver as wired_webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.service import Service
from webdriver_manager.chrome import ChromeDriverManager
from naverplaceapi.mixin.utils import HEADERS, parse_naver_var_in_script_texts
from . import query
from ..tor_manager import tor_manager


def get_value_by_path(data, path):
    keys = path.split('.')  # 경로를 점을 기준으로 분할하여 리스트 생성
    result = data

    for key in keys:
        if isinstance(result, dict) and key in result:
            result = result[key]
        else:
            return None  # 경로를 따라가는 동안 오류 발생 시 None 반환

    return result


def extract_naver_var(html_text):
    soup = BeautifulSoup(html_text, "html.parser", from_encoding="utf-8")
    scripts = soup.find_all("script")

    naver_var = parse_naver_var_in_script_texts(scripts)
    variable_name = 'window.__APOLLO_STATE__'
    pattern = re.compile(rf'\b{re.escape(variable_name)}\s*=\s*(.*?)(\n)')
    match = pattern.search(naver_var)

    if match is None:
        return None
    # 첫페이지
    data = match.group(1)
    json_data = json.loads(data[:-1])
    return json_data


def collect_graphql_responses(driver):
    responses = []
    for request in driver.requests:
        request_url = request.url
        if "graphql" not in request_url:
            continue
        response = utils.decode(
            request.response.body,
            request.response.headers.get("Content-Encoding", "identity")
        )
        response = response.decode("utf8")
        if "too many request" in response:
            tor_manager.new_identity()

        json_data = json.loads(response)
        responses.append(json_data)
    return responses



def extract_graphql_request(driver):
    for request in driver.requests:
        request_url = request.url
        if "graphql" not in request_url:
            continue
        return request
    return None


def wait_for_graphql_response(driver,
                              max_attempts=10,
                              waiting_time=2
                              ):
    current_attempt = 0

    while current_attempt < max_attempts:
        need_to_waiting = False
        for request in driver.requests:
            request_url = request.url
            if "graphql" not in request_url:
                continue
            if request.response is None:
                need_to_waiting = True
                break
        if need_to_waiting == False:
            break
        time.sleep(waiting_time)


def _request_get_fsas_review_like_selenium(driver, place_id, page_no, page_cnt):
    # 수정된 요청 보내기
    import requests
    data = query.get_fsas_reviews.create(place_id, page_no, page_cnt)

    # 요청 새 세션 생성
    s = requests.Session()
    # Selenium Cookie 획득
    cookies = driver.get_cookies()
    for cookie in cookies:
        s.cookies.set(cookie['name'], cookie['value'])

    # 블로그 정보 요청 GraphQL의 헤더 정보를 획득
    request_headers = None
    graphql_request = extract_graphql_request(driver)
    if graphql_request is not None:
        request_headers = graphql_request.headers

    # 이전 요청과 동일한 형식으로 데이터 요청
    response = s.post("https://pcmap-api.place.naver.com/graphql", headers=request_headers, data=json.dumps(data))
    return response


def _request_get_visitor_review_like_selenium(driver, place_id, page_no, page_cnt):
    # 수정된 요청 보내기
    import requests
    data = query.get_visitor_reviews.create(place_id, page_no, page_cnt)

    # 요청 새 세션 생성
    s = requests.Session()
    # Selenium Cookie 획득
    cookies = driver.get_cookies()
    for cookie in cookies:
        s.cookies.set(cookie['name'], cookie['value'])

    # 블로그 정보 요청 GraphQL의 헤더 정보를 획득
    request_headers = None
    graphql_request = extract_graphql_request(driver)
    if graphql_request is not None:
        request_headers = graphql_request.headers

    # 이전 요청과 동일한 형식으로 데이터 요청
    response = s.post("https://pcmap-api.place.naver.com/graphql", headers=request_headers, data=json.dumps(data))
    return response


class ReviewMixin:
    def get_visitor_reviews(self, business_id: str, page_no: int, page_cnt: int, proxies=None):
        data = query.get_visitor_reviews.create(business_id, page_no, page_cnt)
        response = requests.post("https://pcmap-api.place.naver.com/graphql", headers=HEADERS, data=json.dumps(data),
                                 proxies=proxies)
        response.raise_for_status()
        response_data = response.json()
        graphql_data = response_data['data']['visitorReviews']
        if graphql_data is None:
            graphql_data = {}
        # ['visitorReviews']
        graphql_data['business_id'] = business_id
        return graphql_data

    def get_ugc_reviews(self, business_id: str, page_no: int, page_cnt: int, proxies=None):
        data = query.get_ugc_reviews.create(business_id, page_no, page_cnt)
        response = requests.post("https://pcmap-api.place.naver.com/graphql", headers=HEADERS, data=json.dumps(data),
                                 proxies=proxies)
        response.raise_for_status()
        response_data = response.json()
        graphql_data = response_data['data']['restaurant']['fsasReviews']
        graphql_data['business_id'] = business_id
        return graphql_data

    def get_visitor_review_stats(self, business_id: str, proxies=None):
        data = query.get_visitor_review_stats.create(business_id)
        response = requests.post("https://pcmap-api.place.naver.com/graphql", headers=HEADERS, data=json.dumps(data),
                                 proxies=proxies)
        response.raise_for_status()
        response_data = response.json()
        graphql_data = response_data['data']['visitorReviewStats']
        if graphql_data is None:
            return None
        graphql_data['_id'] = graphql_data['id']
        graphql_data['business_id'] = business_id
        return graphql_data

    def get_visitor_review_photos_in_visitor_review_tab(self, store_id: str, page_no: int, page_size: int,
                                                        proxies=None):
        data = query.get_visitor_review_photos_in_visitor_review_tab.create(store_id, page_no, page_size)
        response = requests.post("https://pcmap-api.place.naver.com/graphql", headers=HEADERS, data=json.dumps(data),
                                 proxies=proxies)
        response.raise_for_status()
        response_data = response.json()
        graphql_data = response_data['data']['visitorReviews']
        if graphql_data is None:
            graphql_data = {}
        graphql_data['business_id'] = store_id
        return graphql_data

    def get_visitor_review_theme_lists(self, store_id: str, page_no, page_size, proxies=None):
        data = query.get_visitor_review_theme_lists.create(store_id, page_no, page_size)
        response = requests.post("https://pcmap-api.place.naver.com/graphql", headers=HEADERS, data=json.dumps(data),
                                 proxies=proxies)
        response.raise_for_status()
        response_data = response.json()
        graphql_data = response_data['data']['themeLists']
        graphql_data['business_id'] = store_id

        return graphql_data

    def get_blog_reviews_in_html(self,
                                 business_id: str,
                                 page_no=1,
                                 page_size=10,
                                 use_tor:bool =False,
                                 tor_port:int =9150,
                                 proxies=None):
        page_size = max(min(page_size, 10), 1)

        options = webdriver.ChromeOptions()
        options.add_argument('--disable-dev-shm-usage')
        # options.add_argument('headless')
        options.add_argument('--no-sandbox')

        seleniumwire_options = {}
        if use_tor is True:
            seleniumwire_options['proxy'] = {
                'http': f'socks5://127.0.0.1:{tor_port}',
                'https': f'socks5://127.0.0.1:{tor_port}',
                'no_proxy': 'localhost,127.0.0.1'
            }

        driver = wired_webdriver.Chrome(service=Service(ChromeDriverManager().install()),
                                        options=options,
                                        seleniumwire_options=seleniumwire_options)

        def request_interceptor(request):
            if "cc.naver.com" in request.host:
                request.abort()
            if "sentry" in request.host:
                request.abort()
            if "search.pstatic.net" in request.host:
                request.abort()
            if request.path.endswith(('.png', '.jpg', '.gif', 'jpeg', "css")):
                request.abort()

        driver.request_interceptor = request_interceptor
        url = "https://pcmap.place.naver.com/restaurant/{}/review/ugc?type=photoView".format(business_id)
        driver.get(url)
        # 페이지 로딩을 위해 잠시 대기합니다.
        # JavaScript를 사용하여 스크롤을 페이지의 가장 아래로 내립니다.

        html_text = driver.page_source
        interval_page = 10
        try:
            pagination_element = driver.find_element(By.CLASS_NAME, "fvwqf")
            while pagination_element is not None:
                time.sleep(interval_page)
                pagination_element.click()
                driver.execute_script("window.scrollTo(0, document.body.scrollHeight);")
                pagination_element = driver.find_element(By.CLASS_NAME, "fvwqf")
        except NoSuchElementException as e:
            pass
        except StaleElementReferenceException as e:
            pass

        wait_for_graphql_response(driver)
        graphql_responses = collect_graphql_responses(driver)
        naver_var = extract_naver_var(html_text)

        blogs = []
        for graphql_response in graphql_responses:
            for response in graphql_response:
                blogs_response = get_value_by_path(response, "data.fsasReviews")
                if blogs_response is not None:
                    blogs.append(blogs_response)

        driver.close()
        return {
            'html_text': html_text,
            'blogs': blogs,
            'naver_var': naver_var
        }



        # 안댐...
        # gql_response = _request_get_fsas_review_like_selenium(driver, business_id, page_no, page_size)
        # gql_response.raise_for_status()
        # response_data = gql_response.json()
        # graphql_data = response_data['data']['fsasReviews']
        # graphql_data['business_id'] = business_id
        # return graphql_data

    def get_visitor_reviews_in_html(self,
                                    business_id: str,
                                    page_no=1,
                                    page_size=10,
                                    use_tor:bool = False,
                                    tor_port:int = 9150,
                                    proxies=None):
        page_size = max(min(page_size, 10), 1)

        options = webdriver.ChromeOptions()
        options.add_argument('--disable-dev-shm-usage')
        options.add_argument('headless')
        options.add_argument('--no-sandbox')

        if use_tor is True:
            tor_host = f'socks5://127.0.0.1:{tor_port}'
            options.add_argument(f'--proxy-server={tor_host}')

        driver = wired_webdriver.Chrome(service=Service(ChromeDriverManager().install()), options=options)

        def request_interceptor(request):
            if request.path.endswith(('.png', '.jpg', '.gif', 'jpeg')):
                request.abort()

        driver.request_interceptor = request_interceptor

        url = "https://pcmap.place.naver.com/restaurant/{}/review/ugc?type=photoView".format(business_id)
        driver.get(url)
        time.sleep(4)
        gql_response = _request_get_visitor_review_like_selenium(driver, business_id, page_no, page_size)

        gql_response.raise_for_status()
        response_data = gql_response.json()
        graphql_data = response_data['data']['fsasReviews']
        graphql_data['business_id'] = business_id
        return graphql_data
